# funcionFourier.py
# Programa que muestra la grafica de una funcion
# definida por partes y el desarrollo en series
# de Fourier de la misma, generando la senal senoidal

# Importar librerias
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.widgets import Slider

# Valores para x
xa = np.linspace(-np.pi, np.pi, 100)
xb = np.linspace(-np.pi*3, np.pi*3, 100)

# Valores iniciales para n
init_n = 10

# Generar los valores de la funcion por partes
def funcionPorPartes(x):
	# Lista de valores para y
	global y
	y = []
	
	# Para los valores en x
	for k in x:
		# Intervalos de la funcion
		if k > -np.pi and k < 0:
			y.append(0)
		elif k >= 0 and k < np.pi:
			y.append(np.sin(k))
		else:
			# Cerrar la figura
			y.append(0)
	
	# Devolver el vector de valores
	return y

# Generar la funcion de fourier
def funcionFourier(x, n):
	# Lista de valores para y
	y = []
	
	# Coeficiente a0
	a0 = (1/np.pi)  
	
	# Para cada valor en x
	for t in x:

		# Sumatoria
		suma = 0
		b1= (1/2) * np.sin(t)
		# Para cada valor de n
		for k in range(2, n+1):
			
			suma = suma + (1/np.pi)*(((-1)**k+1 )/( 1 - k**2)) * np.cos(k*t)
		
		# Almacenar valor para y
		y.append(suma+a0+b1)
	
	# Devolver los valores de y
	return y

# Dividir la ventana grafica en sub-graficas (subplot)
# Referencias para cada grafica
fig, ax = plt.subplots(2)

# Funcion original de f(x)
ax[0].plot(xa, funcionPorPartes(xa), color='blue')

# Mostrar titulo
ax[0].set_title("Funcion f(x)")

# Referencia de la segunda grafica, serie de fourier
line, = ax[1].plot(xb, funcionFourier(xb, init_n), color='green')

# Mostrar titulo
ax[1].set_title("Serie de Fourier de f(x)")
ax[1].set_ylim(ymin=-1,ymax=3)

# Ajustar la grafica principal haciendo espacio para el slider
plt.subplots_adjust(left=0.1, right=0.9, top = 0.9, bottom=0.2, wspace=0.4, hspace=0.4)

# Color del slider
axcolor = 'lightgoldenrodyellow'

# Crear un slider horizontal para controlar n
axn = plt.axes([0.25, 0.05, 0.65, 0.03], facecolor=axcolor)
parciales_slider = Slider(
	ax=axn,
	label='Sumas parciales',
	valmin=1,
	valstep=1,
	valmax=100,
	valinit=init_n,
)

# Funcion a ser llamada cuando los valores del slider cambien
def update(val):
	line.set_ydata(funcionFourier(xb, parciales_slider.val))
	fig.canvas.draw_idle()

# Registrar la funcion update del slider
parciales_slider.on_changed(update)

# Mostrar la funcion
plt.show()
